/* 
  Copyright (C) 1998 Norbert Preining and Christofer Edvardsen

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _SMMGR_H_
#define _SMMGR_H_


#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <sys/time.h>
#include <stddef.h>
#include <dirent.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#include <unistd.h>
#include <signal.h>
#include <math.h>
#include <errno.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <stdarg.h>
#ifdef USEGSM
#include <gsm.h>
#endif

extern int isblank(int);

#define VERSION "1.0.5"
#define COPYRIGHT "Copyright (C) 1999 Norbert Preining"

#ifndef CONFIGFILE
#define CONFIGFILE "/etc/smmgr.conf"
#endif

#ifndef LIBDIR
#define LIBDIR "/var/lib/modem-mgr"
#endif

#ifndef SPOOLDIR
#define SPOOLDIR "/var/spool/modem-mgr"
#endif

#ifndef LOCKDIR
#define LOCKDIR "/var/lock"
#endif

/* Characters not allowed in option name */
#define NOTALLOW " =[]"

#define P_NOTFOUND -1
#define E_FILE -2
#define E_SYNTAX -3
#define P_IGNORED -4

#define ATTR_RELEASED 0xfc
#define ATTR_NON_RELEASED 0x1
#define ATTR_NON_CHECKED 0x2

#define PROMPT "smmgr> "

typedef struct msginfo_ {
  int type, pagetime, attr, status, day, hour, min;
  int startpage, address, checksum;
  char id[21];
} msginfo;


typedef int (*PFG)(const char*);

typedef struct {
	char* name;
	char* description;
	PFG pfun;
	int takearg;
	int whichmodems;
} Tcomdef;

#ifdef MACRO
typedef struct _Tmacdef {
	char* name;
	char* expansion;
	struct _Tmacdef *next;
} Tmacdef;
#endif

typedef struct {
	char* modemname;
	int id;
} Tmodemdef;

typedef struct {
	int mcs,mcd,mcf,mcl,mcm,mcp,mcr,mcv,mvd,mct,mcg;
	char mcc[20],mfi[21];
} Tmodemstat;

typedef struct {
	unsigned char *data;
	int length;
	int filled;
} Tpage;

typedef enum { T_UNIDENTIFIED=0, 
	       T_FAX=1, 
	       T_VOICE=2,
               T_DATA=3
} Ttyp;

typedef enum { L_SILENT=0,
		L_ERROR=1,
		L_INFO=2,
		L_DEBUG=3
} logval;

typedef enum {
	M_OK, M_ERROR, M_NORESPONSE
} modemval;

/* Modem Types */
#define USR_MESSAGE_PLUS (0x1)
#define USR_MESSAGE_PRO (0x2)
#define ALL_MODEMS (USR_MESSAGE_PLUS | USR_MESSAGE_PRO)

/* repl.c */
int repl_imon(const char*);
int repl_imoff(const char*);
int repl_play(const char*);
int repl_view(const char*);
int repl_imon(const char*);
int repl_imon_proper(const char*);
int repl_imoff(const char*);
int repl_get(const char*);
int repl_clear(const char*);
int repl_ulg(const char*);
int repl_ulf(const char*);
int repl_recg(const char*);
int repl_recf(const char*);
int repl_play(const char*);
int repl_playg(const char*);
int repl_playf(const char*);
int repl_list(const char*);
int repl_report(const char*);
int repl_clock(const char*);
int repl_freenr(const char*);
int repl_repl(const char*);
int repl_quit(const char*);
int repl_faxdir(const char*);
int repl_voicedir(const char*);
int repl_fax(const char*);
int repl_voice(const char*);
int repl_monitor(const char*);
int repl_remote(const char*);
int repl_answer(const char*);
int repl_password(const char*);
int repl_identity(const char*);
int repl_device(const char*);
int repl_verbose(const char*);
int repl_initstring(const char*);
int repl_libfile(const char*);
int repl_logfile(const char*);
int repl_lockdir(const char*);
int repl_getmsginfo(const char*);
int repl_parseconfig(const char*);
int repl_initdevice(const char*);
int repl_closedevice(const char*);
int repl_help(const char*);
int repl_settings(const char*);
int repl_configfile(const char*);
int repl_savepage(const char*);
int repl_pagesingle(const char*);
int repl_ignore_dle(const char*);
int repl_modemstatus(const char*);
int repl_set_recordtime(const char*);
int repl_imon_on_exit(const char*);
int repl_mail_on_download(const char*);
int repl_mailprog(const char *s);
int repl_viewprog(const char *s);
int repl_faxmail(const char *s);
int repl_voicemail(const char *s);
int repl_out_card(const char *s);
int repl_version(const char *s);
int repl_editlocal(const char*);
int repl_localscript(const char*);
int repl_command(const char*);
int repl_modemtype(const char*);
int repl_tollsaver(const char*);
int repl_loudness(const char*);
#ifdef MACRO
int repl_macro(const char*);
int repl_showmacros(const char*);
#endif

/* main.c */
extern Tcomdef comdef[];
#ifdef MACRO
extern Tmacdef *macdef;
#endif
extern Tmodemdef modemtypes[];
extern Tmodemstat modemstat;
extern Tpage pages[64];
int main(int, char**);
int parseconfigfile();
int start_repl();
int start_processing(int,char**,int*);
int process_one(int,char**,int*);
void exitfunc(int, void*);
void termination_handler(int);
void interrupt_play_handler(int);
void writetolog(char*);
char *rl_gets();
char *command_generator(char*,int);
void initialize_readline();
extern int s_fax, s_voice, s_monitor, s_remote, s_answer, s_verbose, fd;
extern speed_t s_baud;
extern int s_ignore_dle, s_imon_on_exit, s_mail_on_download;
extern char s_faxmail[256], s_voicemail[256], s_mailprog[256];
extern char s_viewprog[256], s_localscript[256];
extern char s_identity[21], s_device[32], s_init[32], s_faxdir[256];
extern char s_voicedir[256], s_libfile[256], s_logfile[256], s_lockdir[256];
extern int s_out_card, s_modemtype, s_tollsaver, s_loudness;
extern char s_faxprefix[256];
extern msginfo *msginfos;
extern int nr_msgs, nr_voice, nr_fax, nr_voice_new, nr_fax_new;
extern int percentfull, inmsg, fullmsg, total_memory;
extern int c_page, c_offset, s_password, have_msginfos, c_firstfree;
extern unsigned char *page;
extern time_t stored_time, timedrift;
extern char lockname[32];
extern struct termios oldiosettings, iosettings;
extern char s_configfile[256];
extern int device_initialized, s_pagesingle;
extern int index2fnr[256];
extern char current_messagename[256];
extern char tmpdir[256];
extern int page_size, getblock_return;

/* modem.c */
int opendev();
int closedev();
char* getblock(char*,int);
int getmsginfo();
unsigned char incoffset(int);
int resetclock(int);
int readclock();
int setiospeed(speed_t);
int get_page(int);
int getmodemstatus();
modemval sendstring(char*, char**, char*);
/*modemval getbintillok(char*, unsigned char**, int*);*/

/* util.c */
int is_voice(int);
int is_fax(int);
int is_not_comment(const char*);
int count_words(const char*);
int line_to_array(char*, char***);
int attr2str(int, char*);
int str2num(char*,int);
int is_true(const char*);
int is_new_or_unknown(int);
int make_rtime(time_t, time_t, time_t, char*, char*);
int make_commentfile(char*, int, char*, char*, char*);
int make_names(char*, char*, char*, int, char*);
int transfer_no_dle_handle(int, int, char*, int, char*);
int transfer_fax(int, int, char*, int, char*);
int transfer_voice(int, int, char*, int, char*);
int transfer_special(int);
int send_files(char*, char**, int, int);
char* dupstr(const char*);
void logit(logval,const char*, ...);
int get_stored_time();
int set_stored_time(time_t);

/* commands.c */
int ulfile(const char*);
int report_msgs();
int listmsg();
int freenr();
int msgtofile();
int newtofile();
int onetofile(int);
int savepagetofile(int, const char*);
int play_soundcard(int);
int playit(const char*);
int view_fax(int);
int viewit(const char*, int);
#ifdef USEGSM
int usr2gsm(char *, int);
#endif

/* local.c */
int local_list(const char*);
int local_play(const char*);
int local_view(const char*);
int local_send(const char*);
int local_remove(const char*);

/* additional */
extern char *xmalloc();

#ifdef USEGSM
/* toast_ulaw */
extern int      ulaw_output   (gsm_signal*, FILE*);
#endif

#endif
